﻿
using System;
using System.Collections.ObjectModel;
using System.Data;

namespace HeaderProblem
{
    /** Interfaccia generica per la mappatura degli oggetti nel DB
     **/
    [Serializable]
    public abstract class ABusinessObjectDB
    {
        public string modulo;
        public string layer;
        public string intestazione;
        public string dettaglio;
        public string footer;

        protected string _nomeTabella;

        protected Collection<object> ultimoOggettoInserito { get; set; }


        public string nomeTabella
        {
            get
            {
                return _nomeTabella;
            }
            set
            {
                _nomeTabella = value;
            }
        }


        /** Metodo da utilizzare per controllare se l'oggetto che si desidera inserire
         *  è già esistente.
         * 
         *  Ritorna vero se l'oggetto esiste.
         *  In oggetto
         *  
         *  Ritorna falso se l'oggetto non esiste.
         **/
        public abstract bool verificaEsistenzaOggetto(ABusinessObjectEntity oggetto);

        /** Metodo da utilizzare per generare il codice necessario per aggiungere un oggetto nel DB.
         * 
         *  Ritorna vero se l'oggetto è stato inserito correttamente.
         *  
         *  Ritorna falso se l'oggetto non è stato inserito correttamente.
         *  Nella classe di gestione dell'errore troveremo la descrizione.
         **/
        public abstract bool aggiungiOggettoDB(ABusinessObjectEntity oggetto);


        /** Metodo da utilizzare per generare il codice necessario per modificare un oggetto nel DB.
         * 
         *  Ritorna vero se l'oggetto è stato modificato correttamente.
         *  
         *  Ritorna falso se l'oggetto non è stato modificato correttamente.
         *  Nella classe di gestione dell'errore troveremo la descrizione.
         **/
        public abstract bool modificaOggettoDB(ABusinessObjectEntity oggetto);


        /** Metodo da utilizzare per generare il codice necessario per eliminare un oggetto nel DB.
         * 
         *  Ritorna vero se l'oggetto è stato eliminato correttamente.
         *  
         *  Ritorna falso se l'oggetto non è stato eliminato correttamente.
         *  Nella classe di gestione dell'errore troveremo la descrizione.
         **/
        public abstract bool eliminaOggettoDB(ABusinessObjectEntity oggetto);


        /** 
         * Metodo da utilizzare per generare il codice necessario per controllare le dipendenze di un oggetto nel DB.
         **/
        public abstract bool controllaDipendenzeElimina(ABusinessObjectEntity oggetto);


        /** Metodo da utilizzare per ritornare il prossimo id della sequenza.
         * 
         *  Ritorna vero se l'id è stato associato correttamente correttamente.
         *  
         *  Ritorna falso se l'id non è stato associato correttamente.
         *  Nella classe di gestione dell'errore troveremo la descrizione.
         **/
        public abstract bool getProssimoIDSequenza(ABusinessObjectEntity oggetto);
        


        /** Metodo da utilizzare per generare il codice necessario per ricerca uno o piu' oggetti nel DB.
         * 
         *  Ritorna vero se la ricerca ha avuto successo.
         *  
         *  Ritorna falso se la ricerca non ha avuto successo.
         *  Nella classe di gestione dell'errore troveremo la descrizione.
         **/
        public abstract DataSet ricercaOggettoDBDS(ABusinessObjectEntity oggetto);


        /** Metodo da utilizzare per generare la condizione SELECT e FROM per l'oggetto nel DB.
         * 
         *  Ritorna la stringa generata.
         **/
        public abstract string getHeaderSelectSQL(ABusinessObjectEntity oggetto);


        /** Metodo da utilizzare per generare la condizione WHERE per l'oggetto nel DB.
         * 
         *  Ritorna la stringa generata.
         **/
        public abstract string getWhereSelectSQL(ABusinessObjectEntity oggetto);

        /** Metodo da utilizzare per generare il codice necessario per ricerca uno o piu' oggetti nel DB.
         * 
         *  Ritorna vero se la ricerca ha avuto successo.
         *  
         *  Ritorna falso se la ricerca non ha avuto successo.
         *  Nella classe di gestione dell'errore troveremo la descrizione.
         **/
        public abstract ObservableCollection<ABusinessObjectEntity> ricercaOggettoDBCol(ABusinessObjectEntity oggetto);


        /** Metodo da utilizzare per interpretare il DataSet ritornato e inserirlo in una ObservaboleCollection.
          * 
          *  Ritorna la collezione se la ricerca ha avuto successo.
          *  
          *  Ritorna NULL se la ricerca non ha avuto successo.
          *  Nella classe di gestione dell'errore troveremo la descrizione.
          **/
        public abstract ObservableCollection<ABusinessObjectEntity> interpretaSQLCollection(DataSet ds);


        /**
         * Funzione che blocca l'oggetto per l'update o il delete.
         * 
         * Ritorna vero se il lock ha avuto successo.
         * Ritorna falso se il lock non ha avuto successo.
         **/
        public abstract bool ricercaOggettoPerUpdate(ABusinessObjectEntity oggetto);

        /**
         * Funzione che ritorna i valori delle proprietà dell'oggetto separati da |.
         **/
        //public abstract ObservableCollection<ABusinessObjectEntity> getItemComboBox(bool ricerca);

        /**
         * Funzione che ritorna i valori delle proprietà dell'oggetto separati da |.
         **/
        public abstract string getValoriOggetto(ABusinessObjectEntity oggetto);

    }
}
