﻿using System;

namespace HeaderProblem
{
    /** Classe astratta generica per la mappatura degli oggetti nella BD. 
     *  Contiene inoltre i metodi per comunicare con la classe responsabile
     *  della realizzazione effettiva della comunicazione con la BD.
     *  
     *  Contiene un metodo di controllo sull'integrità dell'oggetto.
    **/
    [Serializable]
    public abstract class ABusinessObjectEntity
    {

        protected bool   _oggettoCompleto;
        protected bool   _oggettoUnivoco;
        protected bool   _autocommit;
        protected bool   _ricercaEsatta;
        protected string _operazioneDB="";
        protected int    _idsupporto;
        protected string _paramsupportoricerca="";
        protected string _tipoRicerca = "";
        protected string _tipoOggetto = "";
        protected DateTime _timestamp;

       

        /** Metodo da utilizzare per verificare se nell'oggetto 
         *  sono presenti i dati obbligatori.
         *  
         *  Ritorna vero se l'oggetto è completo.
         *  Ritorna falso se l'oggetto non è completo.
         **/
        public abstract bool oggettoIntegro();


        /** Metodo da utilizzare per ritornare il gestore del 
         *  delle connessioni verso il DB
         *  
         *  Ritorna l'oggetto.
         *  Ritorna null se si è verificato un errore.
         **/
        public abstract ABusinessObjectDB getDBHandler();

        /// <summary>
        /// Copia l'oggetto parametro nell'oggetto corrente.
        /// </summary>
        /// <param name="oggettoDaCopiare"></param>
        public abstract void copiaOggetto(ABusinessObjectEntity oggettoDaCopiare);


        public int idsupporto
        {
            get
            {
                return _idsupporto;
            }
            set
            {
                _idsupporto = value;
            }
        }

        public string paramsupportoricerca
        {
            get { return _paramsupportoricerca; }
            set { _paramsupportoricerca = value; }
        }

        public bool oggettoCompleto
        {
            get { return _oggettoCompleto; }
            set
            {
                _oggettoCompleto = value;
            }
        }

        public bool oggettoUnivoco { get; set; }

        /** Metodo da utilizzare per verificare se la classe deve fare il commit al momento 
         *  di un'interazione verso il DB
         *  
         *  Ritorna true se la classe deve fare il commit subito.
         *  Ritorna false se la classe deve delegare alla classe piu' "esterna" il commit.
         **/
        public bool autocommit
        {
            get
            {
                return _autocommit;
            }
            set
            {
                _autocommit = value;
            }
        }

        /// <summary>
        /// Ricerca esatta
        /// <value>
        /// True=ricerca esatta
        /// False=ricerca in sottostringhe
        /// </value>
        /// </summary>
        public bool ricercaEsatta
        {
            get
            {
                return _ricercaEsatta;
            }
            set
            {
                _ricercaEsatta = value;
            }
        }

        /// <summary>
        /// Tipo di operazione da eseguire sul DB, la ricerca è coerente
        /// </summary>
        public string operazioneDB
        {
            get { return _operazioneDB; }
            set
            {
                _operazioneDB = value;
            }
        }

        /** Metodo da utilizzare per verificare se la classe deve fare il commit al momento 
         *  di un'interazione verso il DB
         *  
         *  Ritorna true se la classe deve fare il commit subito.
         *  Ritorna false se la classe deve delegare alla classe piu' "esterna" il commit.
         **/
        public string tipoRicerca
        {
            get
            {
                return _tipoRicerca;
            }
            set
            {
                _tipoRicerca = value;
            }
        }

        /** Metodo da utilizzare per verificare se la classe deve fare il commit al momento 
         *  di un'interazione verso il DB
         *  
         *  Ritorna true se la classe deve fare il commit subito.
         *  Ritorna false se la classe deve delegare alla classe piu' "esterna" il commit.
         **/
        public string tipoOggetto
        {
            get
            {
                return _tipoOggetto;
            }
            set
            {
                _tipoOggetto = value;
            }
        }

        ///<Summary>
        /// Proprietà che contiene l'informazione sull'ultimo aggiornamento del campo.
        /// Permette la gestione dei dati concorrente.
        ///</Summary>
        public DateTime timestamp
        {
            get { return _timestamp; }
            set
            {
                _timestamp = value;
            }
        }

    }
}
