﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using Infragistics.Windows.Controls;
using Infragistics.Windows.DataPresenter;
using BusinessLogicLayer.Definitions;
using Clipboard = System.Windows.Clipboard;
using System.Collections;

namespace HeaderProblem
{
    public class DataGridCommon
    {

        public static RoutedCommand CopyCommand = new RoutedCommand();

        #region Gestione impostazioni di base dei datagrid

        public static void impostaTemaDataGrid(XamDataGrid datagrid)
        {
            datagrid.Theme = "Office2k7Blue";
            datagrid.FontFamily = new FontFamily("Calibri");
            datagrid.FontSize = 12;
            datagrid.FieldSettings.Height = new FieldLength(21);

        }

        public static void abilitaSincronizzazioneConElementoCorrente(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.IsSynchronizedWithCurrentItem = abilitato;
        }

        public static void funzioneSelezioneCella(XamDataGrid datagrid, CellClickAction funzione)
        {
            datagrid.FieldSettings.CellClickAction = funzione;
        }

        public static void funzioneSelezioneLabelCella(XamDataGrid datagrid, LabelClickAction funzione)
        {
            datagrid.FieldSettings.LabelClickAction = funzione;
        }

        public static void abilitaSelezioneMultipla(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.FieldLayoutSettings.SelectionTypeRecord = abilitato ? SelectionType.Extended : SelectionType.Single;
        }

        public static void selezioneCampiVisibiliGriglia(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.FieldLayoutSettings.HeaderPrefixAreaDisplayMode = abilitato ? HeaderPrefixAreaDisplayMode.FieldChooserButton : HeaderPrefixAreaDisplayMode.None;
        }

        public static void abilitaDragAndDrop(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.AllowDrop = abilitato;
        }

        public static void abilitaAggiuntaRecord(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.FieldLayoutSettings.AllowAddNew = abilitato;
            datagrid.FieldLayoutSettings.AddNewRecordLocation = AddNewRecordLocation.OnTopFixed;
        }

        public static void abilitaModificaRecord(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.FieldSettings.AllowEdit = abilitato;
        }

        public static void abilitaEliminazioneRecord(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.FieldLayoutSettings.AllowDelete = abilitato;
        }

        public static void abilitaAutoGenerazioneCampi(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.FieldLayoutSettings.AutoGenerateFields = abilitato;
        }

        public static void abilitaAutoFit(XamDataGrid datagrid, bool abilitato)
        {
            datagrid.AutoFit = abilitato;
        }

        public static void abilitaGroupBy(XamDataGrid datagrid, bool abilitato)
        {
            if (abilitato)
            {
                datagrid.FieldSettings.AllowGroupBy = abilitato;
                datagrid.GroupByAreaLocation = GroupByAreaLocation.AboveDataArea;
                datagrid.GroupByAreaMode = GroupByAreaMode.MultipleFieldLayoutsCompact;
                datagrid.IsGroupByAreaExpanded = false;
                datagrid.GroupByArea.Prompt1 = "Area di raggruppamento";
                datagrid.GroupByArea.Prompt2 = "Trascina un campo"; 
            }
            else
            {
                datagrid.FieldSettings.AllowGroupBy = abilitato;
                datagrid.GroupByAreaLocation = GroupByAreaLocation.None;
            }
        }

        public static void abilitaFiltriDataGrid(XamDataGrid datagrid, bool abilita)
        {
            if (abilita)
            {
                datagrid.FieldSettings.AllowRecordFiltering = true;
                datagrid.FieldSettings.FilterClearButtonVisibility = Visibility.Visible;
                datagrid.FieldLayoutSettings.FilterUIType = FilterUIType.LabelIcons;
                datagrid.FieldLayoutSettings.DataRecordSizingMode = DataRecordSizingMode.SizedToContentAndFixed;
                datagrid.FieldLayoutSettings.RecordFiltersLogicalOperator = LogicalOperator.And;
                datagrid.FieldLayoutSettings.FilterClearButtonLocation = FilterClearButtonLocation.RecordSelector;
            }
            else
            {
                datagrid.FieldSettings.AllowRecordFiltering = false;
            }
        }

        public static void abilitaCopiaIncolla(XamDataGrid datagrid, RoutedCommand CopyCommandP)
        {
            var copyCommandBinding = new CommandBinding(CopyCommandP, ExecutedCopyCommand);
            datagrid.CommandBindings.Add(copyCommandBinding);
        }

        public static void abilitaLayerErrorDataGrid(XamDataGrid datagrid)
        {
            datagrid.FieldLayoutSettings.SupportDataErrorInfo = SupportDataErrorInfo.RecordsAndCells;
            datagrid.FieldLayoutSettings.DataErrorDisplayMode = DataErrorDisplayMode.ErrorIconAndHighlight;
        }

        #endregion

        #region Generazione campi della griglia

        public static void generaCampiDataGrid(IList<FieldSDB> campiVisibili, DataPresenterBase datagrid)
        {
            //Svuoto tutti i campi.
            datagrid.FieldLayouts[0].Fields.Clear();
            datagrid.FieldLayouts[0].SummaryDefinitions.Clear();

            for (int i = 0; i < campiVisibili.Count; i++)
            {
                if (!campiVisibili[i].unboundCampo)
                {
                    var field = new Field
                    {
                        Name = campiVisibili[i].nomeCampo,
                        Label = campiVisibili[i].labelCampo,
                        Visibility = campiVisibili[i].visibilitaCampo
                    };

                    if (campiVisibili[i].dimensioneCampo != 0.0)
                    {
                        field.Width = new FieldLength(campiVisibili[i].dimensioneCampo);
                    }

                    if (campiVisibili[i].tipoCampo != null)
                    {
                        field.DataType = campiVisibili[i].tipoCampo;
                        field.Settings.EditAsType = campiVisibili[i].tipoCampo;
                    }

                    if (!string.IsNullOrEmpty(campiVisibili[i].stileCellaCampo))
                    {
                        field.Settings.CellValuePresenterStyle = (Style)datagrid.TryFindResource(campiVisibili[i].stileCellaCampo);
                    }

                    if (!string.IsNullOrEmpty(campiVisibili[i].editoreCampo))
                    {
                        field.Settings.EditorStyle = (Style)datagrid.TryFindResource(campiVisibili[i].editoreCampo);
                    }

                    if (campiVisibili[i].solaLettura)
                    {
                        field.Settings.EditorStyle = (Style)datagrid.TryFindResource("ReadOnlyFieldStyle");
                        field.Settings.AllowEdit = false;
                    }

                    field.IsScrollTipField = campiVisibili[i].toolTipScroll;

                    if (campiVisibili[i].campoGroupBy)
                    {
                        field.IsExpandable = true;
                    }

                    datagrid.FieldLayouts[0].Fields.Add(field);
                }
                else
                {
                    var pp = new PropertyPath(campiVisibili[i].bindingCampo);

                    var ufield = new UnboundField
                    {
                        Name = campiVisibili[i].nomeCampo,
                        Label = campiVisibili[i].labelCampo,
                        Visibility = campiVisibili[i].visibilitaCampo,
                        BindingPath = pp,
                        BindingMode = BindingMode.TwoWay
                    };


                    if (campiVisibili[i].dimensioneCampo != 0)
                    {
                        ufield.Width = new FieldLength(campiVisibili[i].dimensioneCampo);
                    }

                    if (campiVisibili[i].tipoCampo != null)
                    {
                        ufield.DataType = campiVisibili[i].tipoCampo;
                        ufield.Settings.EditAsType = campiVisibili[i].tipoCampo;
                    }

                    if (!string.IsNullOrEmpty(campiVisibili[i].stileCellaCampo))
                    {
                        ufield.Settings.CellValuePresenterStyle = (Style)datagrid.TryFindResource(campiVisibili[i].stileCellaCampo);
                    }

                    if (!string.IsNullOrEmpty(campiVisibili[i].editoreCampo))
                    {
                        ufield.Settings.EditorStyle = (Style)datagrid.TryFindResource(campiVisibili[i].editoreCampo);

                    }

                    if (campiVisibili[i].solaLettura)
                    {
                        ufield.Settings.EditorStyle = (Style)datagrid.TryFindResource("ReadOnlyFieldStyle");
                        ufield.Settings.AllowEdit = false;
                    }

                    ufield.IsScrollTipField = campiVisibili[i].toolTipScroll;

                    if (campiVisibili[i].campoGroupBy)
                    {
                        ufield.IsExpandable = true;
                    }

                    datagrid.FieldLayouts[0].Fields.Add(ufield);
                }
            }
        }

        #endregion

        #region Controlli vari sugli elementi della griglia

        /// <summary>
        /// Controlla se esistono elementi selezionati nella griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        /// <returns></returns>
        public static bool checkElementiSelezionati(DataPresenterBase datagrid)
        {
            return datagrid.SelectedItems.Records.Count != 0;
        }

        /// <summary>
        /// Controlla se esistono elementi nella griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        /// <returns></returns>
        public static bool checkElementiPresentiGriglia(DataPresenterBase datagrid)
        {
            return datagrid.Records.Count != 0;
        }

        #endregion

        #region Utilita (N°record selezionati, ...)

        public static int getNumeroElementiEsclusiFiltro(XamDataGrid datagrid)
        {
            return datagrid.RecordManager.GetFilteredOutDataRecords().ToList().Count;
        }

        public static int getNumeroElementiFiltrati(XamDataGrid datagrid)
        {
            return datagrid.RecordManager.GetFilteredInDataRecords().ToList().Count;
        }

        public static int getNumeroElementiSelezionati(XamDataGrid datagrid)
        {
            return datagrid.SelectedItems.Records.Count;
        }

        public static int getNumeroElementiDataGrid(XamDataGrid datagrid)
        {
            return datagrid.Records.Count;
        }

        #endregion

        #region Gestione ordinamento della griglia

        public static void impostaOrdinamento(XamDataGrid datagrid, string nomeCampo)
        {
            var fsd = new FieldSortDescription
                          {
                              FieldName = nomeCampo,
                              Direction = ListSortDirection.Ascending
                          };

            if (datagrid.FieldLayouts.Count == 1)
            {
                datagrid.FieldLayouts[0].SortedFields.Add(fsd);
            }
                
            else
            {
                //Definizione dell'ordinamento.
                var fieldlayout = new FieldLayout();
                fieldlayout.SortedFields.Add(fsd);
                datagrid.FieldLayouts.Add(fieldlayout);
            }
        }
        public static void impostaComparerOrdinamento(XamDataGrid datagrid, string nomeCampo, IComparer comparer)
        {
            if (datagrid.FieldLayouts.Count == 1)
            {
                var field = datagrid.FieldLayouts[0].Fields[nomeCampo];
                if(field!=null)
                    field.Settings.SortComparer = comparer;
            }
        }
        public static void impostaOrdinamento(XamDataGrid datagrid, string nomeCampo, bool groupby)
        {
            var fsd = new FieldSortDescription
            {
                FieldName = nomeCampo,
                IsGroupBy   = true,
                Direction = ListSortDirection.Ascending
            };
            if (datagrid.FieldLayouts.Count == 1)
            {
                datagrid.FieldLayouts[0].SortedFields.Add(fsd);
            }
            else
            {
                //Definizione dell'ordinamento.
                var fieldlayout = new FieldLayout();
                fieldlayout.SortedFields.Add(fsd);
                datagrid.FieldLayouts.Add(fieldlayout);
            }
        }
        public static void impostaOrdinamentoInverso(XamDataGrid datagrid, string nomeCampo)
        {
            var fsd = new FieldSortDescription
            {
                FieldName = nomeCampo,
                Direction = ListSortDirection.Descending
            };
            if (datagrid.FieldLayouts.Count == 1)
            {
                datagrid.FieldLayouts[0].SortedFields.Add(fsd);
            }
            else
            {
                //Definizione dell'ordinamento.
                var fieldlayout = new FieldLayout();
                fieldlayout.SortedFields.Add(fsd);
                datagrid.FieldLayouts.Add(fieldlayout);
            }
        }
        public static void ordinaDataGrid(DataPresenterBase datagrid)
        {
            if (datagrid.RecordManager.GetFilteredInDataRecords().Count() == 0) return;
            var temp = new FieldSortDescription[datagrid.DefaultFieldLayout.SortedFields.Count];
            datagrid.DefaultFieldLayout.SortedFields.CopyTo(temp, 0);
            datagrid.DefaultFieldLayout.SortedFields.Clear();
            for (int i = 0; i < temp.Length; i++)
            {
                datagrid.DefaultFieldLayout.SortedFields.Add(temp[i]);
            }
        }

        #endregion

        #region Gestione Header campi

        public static void abilitaHeaderDataGrid(XamDataGrid datagrid)
        {
            var stile = datagrid.TryFindResource("headerDataGrid") as Style;

            if (stile != null)
            {
                datagrid.FieldLayoutSettings.HeaderLabelAreaStyle = datagrid.TryFindResource("headerDataGrid") as Style;
            }
        }


        #endregion

        #region Gestione sommario

        public static void impostaSommario(XamDataGrid datagrid, string nomeCampo, string tipoSommario)
        {
            var sommario = new SummaryDefinition();

            switch(tipoSommario)
            {
                case DataGridSummaryType.Tipo.somma:
                    var sommaElementiSummary = new DataGridSumSummary();
                    SummaryCalculator.Register(sommaElementiSummary);
                    sommario.Calculator = sommaElementiSummary;
                    break;
                case DataGridSummaryType.Tipo.media:
                    sommario.Calculator = SummaryCalculator.Average;
                    break;
                case DataGridSummaryType.Tipo.conteggio:
                    var conteggioElementiSummary = new DataGridCounterSummary();
                    SummaryCalculator.Register(conteggioElementiSummary);
                    sommario.Calculator = conteggioElementiSummary;
                    //sommario.Calculator = SummaryCalculator.Count;
                    break;
                case DataGridSummaryType.Tipo.elementiselezionati:
                    var elementiSelezionatiSummary = new DataGridElementiSelezionatiSummary();
                    SummaryCalculator.Register(elementiSelezionatiSummary);
                    sommario.Calculator = elementiSelezionatiSummary;
                    break;
                default:
                    break;
            }

            sommario.SourceFieldName = nomeCampo;

            if (datagrid.FieldLayouts.Count == 1)
            {
                datagrid.FieldLayouts[0].SummaryDefinitions.Add(sommario);
            }
            else
            {
                //Definizione del sommario
                var fieldlayout = new FieldLayout();
                fieldlayout.SummaryDefinitions.Add(sommario);
                datagrid.FieldLayouts.Add(fieldlayout);
            }
        }

        public static void abilitaSommario(XamDataGrid datagrid)
        {
             datagrid.FieldSettings.AllowSummaries = false;
             datagrid.FieldSettings.SummaryUIType = SummaryUIType.SingleSelect;
             datagrid.FieldSettings.SummaryDisplayArea = SummaryDisplayAreas.BottomFixed;
        }

        #endregion

        #region Gestione liste specifiche ritornate dalla griglia


        /// <summary>
        /// Ritorna la lista degli elementi della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        /// <returns>Lista con gli elementi</returns>
        public static ObservableCollection<ABusinessObjectEntity> getListaElementiDataGrid(DataPresenterBase datagrid)
        {
            var lista = new ObservableCollection<ABusinessObjectEntity>();

            //Copio la collezione degli elementi selezionati.
            for (var i = 0; i < datagrid.Records.Count; i++)
            {
                var itemSel = ((DataRecord)(datagrid.Records[i])).DataItem as ABusinessObjectEntity;
                if (itemSel == null) continue;
                lista.Add(itemSel);
            }

            return lista;
        }

        /// <summary>
        /// Ritorna la lista degli elementi della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        /// <returns>Lista con gli elementi</returns>
        public static ObservableCollection<ABusinessObjectEntity> getListaElementiFiltratiDataGrid(DataPresenterBase datagrid)
        {
            var lista = new ObservableCollection<ABusinessObjectEntity>();
            var listaElementi = datagrid.RecordManager.GetFilteredInDataRecords().ToArray();

            //Copio la collezione degli elementi selezionati.
            for (var i = 0; i < listaElementi.Length; i++)
            {
                var itemSel = (listaElementi[i]).DataItem as ABusinessObjectEntity;
                if (itemSel == null) continue;
                lista.Add(itemSel);
            }

            return lista;
        }

        /// <summary>
        /// Ritorna la lista degli elementi selezionati della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        /// <returns>Lista con gli elementi selezionati</returns>
        public static ObservableCollection<ABusinessObjectEntity> getListaElementiSelezionatiDataGrid(DataPresenterBase datagrid)
        {
            var lista = new ObservableCollection<ABusinessObjectEntity>();

            //Copio la collezione degli elementi selezionati.
            for (var i = 0; i < datagrid.SelectedItems.Records.Count; i++)
            {
                var itemSel = ((DataRecord)(datagrid.SelectedItems.Records[i])).DataItem as ABusinessObjectEntity;
                if (itemSel == null) continue;
                lista.Add(itemSel);
            }

            return lista;
        }

        #endregion

        #region Gestione delle selezioni degli elementi della griglia

        public static void sostituisciListaRiselezionaElemento(DataPresenterBase datagrid, ObservableCollection<ABusinessObjectEntity> listaOggetti)
        {
            if (datagrid.SelectedItems.Count() > 0)
            {
                //Memorizzo l'elemento attualmente selezionato.
                var elementoSelezionato = ((DataRecord)datagrid.SelectedItems.Records[0]).DataItem as ABusinessObjectEntity;

                //Aggiorno la lista dei dati con la nuova lista.
                datagrid.DataSource = listaOggetti;

                //Riseleziono l'oggetto selezionato memorizzato in precedenza.
                var chk = selezionaElementoLista(datagrid, elementoSelezionato);

                //Se non ho selezionato l'elemento selezionato prima della sostituzione seleziono il primo elemento visibile della lista.
                if(!chk)
                {
                    selezionaPrimoElementoVisibile(datagrid);
                }
            }
            else
            {
                //Aggiorno la lista dei dati con quelli nuovi ricevuti.
                datagrid.DataSource = listaOggetti;
            }
        }


        /// <summary>
        /// Seleziona tutti gli elementi della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare gli elementi</param>
        public static void selezionaTuttiElementi(DataPresenterBase datagrid)
        {
            deselezionaElementi(datagrid);

            IEnumerable<DataRecord> recordFiltrati = datagrid.RecordManager.GetFilteredInDataRecords();
            datagrid.SelectedItems.Records.AddRange(recordFiltrati.ToArray());
        }

        /// <summary>
        /// Seleziona il primo elemento della lista.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare gli elementi</param>
        public static bool selezionaPrimoElemento(DataPresenterBase datagrid)
        {
            try
            {
                return datagrid.ExecuteCommand(DataPresenterCommands.RecordFirstOverall);
            }
            catch (Exception)
            {
                return false;
            }
        }

        /// <summary>
        /// Seleziona il primo elemento visibile della lista.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare gli elementi</param>
        public static bool selezionaPrimoElementoVisibile(DataPresenterBase datagrid)
        {
            try
            {
                return datagrid.ExecuteCommand(DataPresenterCommands.RecordFirstDisplayed);
            }
            catch(Exception)
            {
                return false;
            }
        }

        /// <summary>
        /// Seleziona l'ultimo elemento visibile della lista.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare gli elementi</param>
        public static bool selezionaUltimoElementoVisibile(DataPresenterBase datagrid)
        {
            return datagrid.ExecuteCommand(DataPresenterCommands.RecordLastDisplayed);
        }

        /// <summary>
        /// Seleziona un elemento della lista dato un oggetto.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare gli elementi</param>
        /// <param name="oggetto">Oggetto da selezionare</param>
        /// <returns></returns>
        public static bool selezionaElementoLista(DataPresenterBase datagrid, ABusinessObjectEntity oggetto)
        {
            deselezionaElementi(datagrid);

            var chk = false;
            //Itero la lista degli elementi
            for (var i = 0; i < datagrid.Records.Count; i++)
            {
                var itemSel = ((DataRecord)(datagrid.Records[i])).DataItem as ABusinessObjectEntity;
                if (itemSel == null) continue;
                if (itemSel.Equals(oggetto))
                {
                    datagrid.Records[i].IsSelected = true;
                    datagrid.Records[i].IsActive = true;
                    datagrid.BringRecordIntoView(datagrid.Records[i]);

                    chk = true;
                    return chk;
                }
                datagrid.Records[i].IsSelected = false;
                datagrid.Records[i].IsActive = false;
            }

            return chk;
        }

        public static int indiceElementoSelezionato(DataPresenterBase datagrid)
        {
            if (datagrid.Records.Count > 0)
            {
                if (datagrid.ActiveRecord != null)
                {
                    return datagrid.ActiveRecord.Index;
                }
                return 0;
            }
            return -1;
        }

        public static int selezionaElementoDaIndice(DataPresenterBase datagrid, int indice)
        {
            deselezionaElementi(datagrid);

            if (indice == -1)
            {
                return -1;
            }

            //Se la lista è vuota ritorno -1.
            if (datagrid.Records.Count == 0)
            {
                return -1;
            }
            //Se elimino l'ultimo elemento
            if (datagrid.Records.Count == indice)
            {
                datagrid.ExecuteCommand(DataPresenterCommands.RecordLastDisplayed);
                return 0;
            }
            //Altri casi.
            if (datagrid.Records.Count > indice)
            {
                selezionaElementoPrecedente(datagrid);
                return 1;
            }
            return -1;
        }

        /// <summary>
        /// Seleziona l'elemento precedente.
        /// </summary>
        /// <param name="datagrid"></param>
        /// <returns></returns>
        public static bool selezionaElementoPrecedente(DataPresenterBase datagrid)
        {
            return datagrid.ExecuteCommand(DataPresenterCommands.RecordAbove);
        }

        /// <summary>
        /// Seleziona l'elemento successivo.
        /// </summary>
        /// <param name="datagrid"></param>
        /// <returns></returns>
        public static bool selezionaElementoSuccessivo(DataPresenterBase datagrid)
        {
            return datagrid.ExecuteCommand(DataPresenterCommands.RecordNext);
        }

        /// <summary>
        /// Seleziona il prossimo elemento disponibile.
        /// </summary>
        /// <param name="datagrid"></param>
        /// <returns></returns>
        public static bool selezionaProssimoElemento(DataPresenterBase datagrid)
        {
            var chk = selezionaElementoSuccessivo(datagrid);

            if (!chk)
            {
                chk = selezionaElementoPrecedente(datagrid);

                return chk;
            }

            return chk;
        }



        #endregion

        #region Gestione degli spostamenti della selezione.

        /// <summary>
        /// Seleziona il primo elemento della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare</param>
        public static void vaiPrimoElementoLista(DataPresenterBase datagrid)
        {
            datagrid.ExecuteCommand(DataPresenterCommands.RecordFirstOverall);
        }

        /// <summary>
        /// Seleziona il primo elemento visibile della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare</param>
        public static void vaiPrimoElementoVisibileLista(DataPresenterBase datagrid)
        {
            datagrid.ExecuteCommand(DataPresenterCommands.RecordFirstDisplayed);
        }

        /// <summary>
        /// Seleziona l'ultimo elemento della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare</param>
        public static void vaiUltimoElementoLista(DataPresenterBase datagrid)
        {
            datagrid.ExecuteCommand(DataPresenterCommands.RecordLastOverall);
        }

        /// <summary>
        /// Seleziona l'ultimo  elemento visibile della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia dove selezionare</param>
        public static void vaiUltimoElementoVisibileLista(DataPresenterBase datagrid)
        {
            datagrid.ExecuteCommand(DataPresenterCommands.RecordLastDisplayed);
        }

        #endregion

        #region Gestione delle deselezioni, cancellazioni e del reset della griglia

        /// <summary>
        /// Ritorna la lista degli elementi selezionati della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        /// <returns>Lista con gli elementi selezionati</returns>
        public static bool eliminaElementiSelezionatiDataGrid(DataPresenterBase datagrid)
        {
            datagrid.SelectedItems.Records.Clear();

            return true;
        }

        /// <summary>
        /// Disattiva il record attivo della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        public static void disattivaElemento(DataPresenterBase datagrid)
        {
            datagrid.ActiveRecord = null;
            deselezionaElementi(datagrid);
        }

        /// <summary>
        /// Deseleziona gli elementi della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        public static void deselezionaElementi(DataPresenterBase datagrid)
        {
            datagrid.ExecuteCommand(DataPresenterCommands.ClearAllSelected);
        }

        /// <summary>
        /// Resetta la griglia ed imposta la nuova lista.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        /// <param name="lista">Nuova lista da impostare</param>
        public static void resettaGriglia(DataPresenterBase datagrid, ObservableCollection<ABusinessObjectEntity> lista)
        {
            try
            {
                datagrid.DataSource = lista;
            }
            catch(Exception)
            {
                return;
            }
        }

        #endregion

        #region Gestione errori dei dati nella griglia.

        /// <summary>
        /// Controlla se nella griglia sono presenti dei record con errori.
        /// </summary>
        /// <param name="datagrid">Griglia dove ricercare</param>
        /// <returns>Vero se tutto ok, falso se un record contiene un errore.</returns>
        public static bool chkDatiConErroreDataGrid(DataPresenterBase datagrid)
        {
            foreach (DataRecord Record in datagrid.RecordManager.Current)
            {
                // Traverse each Cell
                foreach (Cell currentCell in Record.Cells)
                {
                    // Get the Cell's CellValuePresenter
                    CellValuePresenter cvp = CellValuePresenter.FromCell(currentCell);

                    // Interrogate for errors, add to collection if necessary
                    if (cvp != null && cvp.HasDataError)
                    {
                        //string ErrorMsg = cvp.DataError.ToString();
                        return false;
                    }
                }
            }

            return true;
        }


        public static void controllaValiditaDati(IDataErrorInfo oggetto, string nomeCampo, ValueEditor editore)
        {
            if (oggetto != null)
            {
                string errore = oggetto[nomeCampo];

                if (!string.IsNullOrEmpty(errore))
                {
                    editore.BorderBrush = Brushes.Red;

                    if (editore.ToolTip != null)
                    {
                        editore.Tag = editore.ToolTip;
                    }
                    editore.ToolTip = errore;
                }
                else
                {
                    if (editore.Tag != null)
                    {
                        editore.ToolTip = null;
                        editore.Tag = null;
                    }
                    else
                    {
                        editore.ToolTip = null;
                    }
                    editore.BorderBrush = null;
                }
            }
        }

        public static void setWarningDati(bool warning, string tooltip, ValueEditor editore)
        {
            if (warning)
            {
                editore.BorderBrush = Brushes.Brown;

                if (editore.ToolTip != null)
                {
                    editore.Tag = editore.ToolTip;
                }
                editore.ToolTip = tooltip;
            }
            else
            {
                if (editore.Tag != null)
                {
                    editore.ToolTip = editore.Tag;
                    editore.Tag = null;
                }
                else
                {
                    editore.ToolTip = null;
                }
                editore.BorderBrush = null;
            }
            
        }
        #endregion

        #region Gestione personalizzazioni della griglia

        /// <summary>
        /// Salva le personalizzazioni dei DataGrid.
        /// </summary>
        /// <param name="datagrid">DataGrid.</param>
        /// <param name="nomeFile">Nome del file dove salvare.</param>
        public static void salvaPersonalizzazioneDG(DataPresenterBase datagrid, string nomeFile)
        {
            //string nomeFileCompleto = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) + "/Layout/" + nomeFile;
            string nomeFileCompleto = sessione.directorytiforma + "\\Layout\\" + nomeFile;
            
            using (var fs = new FileStream(nomeFileCompleto, FileMode.Create, FileAccess.Write))
            {
                try
                {
                    datagrid.SaveCustomizations(fs);
                }
                catch(Exception)
                {
                    File.Delete(nomeFileCompleto);
                }
            }
        }

        /// <summary>
        /// Salva le personalizzazioni dei DataGrid.
        /// </summary>
        /// <param name="datagrid">DataGrid.</param>
        /// <param name="tipo">Tipo di customizzazione da eliminare.</param>
        public static void eliminaPersonalizzazioneDG(DataPresenterBase datagrid, CustomizationType tipo)
        {
            datagrid.ClearCustomizations(tipo);
        }

        /// <summary>
        /// Carica la personalizzazione del DataGrid.
        /// </summary>
        /// <param name="datagrid">Datagrid</param>
        /// <param name="nomeFile">Nome del file salvato.</param>
        public static void caricaPersonalizzazioniDG(DataPresenterBase datagrid, string nomeFile)
        {
            //string nomeFileCompleto = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) + "/Layout/" + nomeFile;
            string nomeFileCompleto = sessione.directorytiforma + "\\Layout\\" + nomeFile;
            //Carico il layout se esiste il file.
            if (File.Exists(nomeFileCompleto))
            {
                using (var fs = new FileStream(nomeFileCompleto, FileMode.Open, FileAccess.Read))
                {
                    try
                    {
                        datagrid.LoadCustomizations(fs);
                    }
                    catch(Exception)
                    {
                        File.Delete(nomeFileCompleto);
                    }
                }
            }
        }

        #endregion

        #region Gestione filtri della Griglia

        /// <summary>
        /// Resetta i filtri impostati della griglia.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        public static void resettaFiltri(XamDataGrid datagrid)
        {
            datagrid.FieldLayouts[0].RecordFilters.Clear();
            datagrid.FieldLayouts[0].RecordFilters.Refresh();
        }

        /// <summary>
        /// Imposta il nuovo filtro alla griglia.
        /// </summary>
        /// <param name="datagrid">Datagrid dove aggiungere il filtro.</param>
        /// <param name="nomeCampo">Nome del campo da filtrare</param>
        /// <param name="valore">Valore da filtrare</param>
        /// <param name="comparatore">Tipo di confronto</param>
        public static void filtraDatiInternoDG(XamDataGrid datagrid, string nomeCampo, string valore, ComparisonOperator comparatore)
        {
            // Get the record filter collection from the data grid.
            RecordFilterCollection recordFilterCollection = datagrid.FieldLayouts[0].RecordFilters;

            if (string.IsNullOrEmpty(valore)) return;

            var recordFilter = new RecordFilter
                                   {
                                       FieldName = nomeCampo
                                   };
            recordFilter.Conditions.Add(new ComparisonCondition(comparatore, valore));
            //recordFilter.Conditions.LogicalOperator = LogicalOperator.And;

            // Aggiungo il nuovo filtro
            recordFilterCollection.Add(recordFilter);
        }

        #endregion

        #region Gestione copia/incolla

        /// <summary>
        /// Gestisce il copia/incolla degli elementi della griglia.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public static void ExecutedCopyCommand(object sender, ExecutedRoutedEventArgs e)
        {
            var myGrid = e.Source as XamDataGrid;

            if (myGrid != null)
            {
                if (myGrid.SelectedItems.Cells.Count > 0)
                    CopySelectedCells(myGrid);
                else if (myGrid.SelectedItems.Records.Count > 0)
                    CopySelectedRecords(myGrid);
                else if (myGrid.SelectedItems.Fields.Count > 0)
                    CopySelectedFields(myGrid);
            }
        }

        /// <summary>
        /// Copia le celle selezionate.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        public static void CopySelectedCells(XamDataGrid datagrid)
        {
            string myInternalClipboard = null;
            int currentRecordLevel = 0;
            Infragistics.Windows.DataPresenter.SelectedCellCollection mySelectedCells = datagrid.SelectedItems.Cells;

            foreach (Cell myCell in mySelectedCells)
            {
                if (myCell.Field.Visibility == Visibility.Hidden || myCell.Field.Visibility == Visibility.Collapsed || myCell.Value == null)
                {
                    continue;
                }
                if (myInternalClipboard == null)
                {
                    currentRecordLevel = myCell.Record.DataItemIndex;
                    myInternalClipboard = myCell.Value.ToString().Trim();
                }
                else if (myCell.Record.DataItemIndex == currentRecordLevel)
                    myInternalClipboard = myInternalClipboard + "\t" + myCell.Value.ToString().Trim();
                else
                {
                    currentRecordLevel = myCell.Record.DataItemIndex;
                    myInternalClipboard = myInternalClipboard + "\r\n" + myCell.Value.ToString().Trim();
                }
            }
            myInternalClipboard = myInternalClipboard + "\r\n";
            Clipboard.Clear();
            Clipboard.SetText(myInternalClipboard);
        }

        /// <summary>
        /// Copia le righe selezionate.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        public static void CopySelectedRecords(XamDataGrid datagrid)
        {
            string myInternalClipboard = null;
            bool IsNewRecord = true;
            Infragistics.Windows.DataPresenter.SelectedRecordCollection mySelectedRecords = datagrid.SelectedItems.Records;

            foreach (DataRecord myRecord in mySelectedRecords)
            {
                foreach (Cell myCell in myRecord.Cells)
                {
                    if (myCell.Field.Visibility == Visibility.Hidden || myCell.Field.Visibility == Visibility.Collapsed || myCell.Value == null)
                    {
                        continue;
                    }
                    if (myInternalClipboard == null || IsNewRecord)
                    {
                        myInternalClipboard = myInternalClipboard + myCell.Value.ToString().Trim();
                        IsNewRecord = false;
                    }
                    else
                        myInternalClipboard = myInternalClipboard + "\t" + myCell.Value.ToString().Trim();
                }
                IsNewRecord = true;
                myInternalClipboard = myInternalClipboard + "\r\n";
            }
            Clipboard.Clear();
            if (myInternalClipboard != null) Clipboard.SetText(myInternalClipboard);
        }

        /// <summary>
        /// Copia i campi selezionati.
        /// </summary>
        /// <param name="datagrid">Griglia</param>
        public static void CopySelectedFields(XamDataGrid datagrid)
        {
            string myInternalClipboard = null;
            bool IsNewRecord = true;
            Infragistics.Windows.DataPresenter.SelectedFieldCollection mySelectedFields = datagrid.SelectedItems.Fields;

            foreach (DataRecord myRecord in datagrid.Records)
            {
                foreach (Field myField in mySelectedFields)
                {
                    if (myField.Visibility == Visibility.Hidden || myField.Visibility == Visibility.Collapsed || myRecord.Cells[myField].Value == null)
                    {
                        continue;
                    }

                    if (myInternalClipboard == null || IsNewRecord)
                    {
                        myInternalClipboard = myInternalClipboard + myRecord.Cells[myField].Value.ToString().Trim();
                        IsNewRecord = false;
                    }
                    else
                    {
                        myInternalClipboard = myInternalClipboard + "\t" + myRecord.Cells[myField].Value.ToString().Trim();
                    }
                }
                IsNewRecord = true;
                myInternalClipboard = myInternalClipboard + "\r\n";
            }
            Clipboard.Clear();
            if (myInternalClipboard != null) Clipboard.SetText(myInternalClipboard);
        }

        #endregion

        #region Gestione esportazione in excel

        public static void esportaGrigliaExcel(XamDataGrid datagrid, string nomeFile)
        {
            var xamDataGridExcelExporter = new DataGridExcelExporter();

            xamDataGridExcelExporter.Export(datagrid, nomeFile);
        }

        #endregion

        public static void setProprietaPannelliModifica(XamDataGrid datagrid)
        {
            //Settaggi del datagrid generico
            impostaTemaDataGrid(datagrid);

            //Abilito l'auto fit delle colonne
            abilitaAutoFit(datagrid, true);

            //Abilito la selezione con l'elemento corrente
            //abilitaSincronizzazioneConElementoCorrente(datagrid, true);

            //Abilito il Drag And Drop
            abilitaDragAndDrop(datagrid, false);

            //Imposto la funzione per al momento della selezione di una cella
            funzioneSelezioneCella(datagrid, CellClickAction.EnterEditModeIfAllowed);

            //Abilita la selezione multipla degli elementi.
            abilitaSelezioneMultipla(datagrid, true);

            //Disabilito la generazione automatica dei campi nella griglia
            abilitaAutoGenerazioneCampi(datagrid, false);

            //Abilito il sommario
            abilitaSommario(datagrid);

            //Abilito il group by nella griglia
            abilitaGroupBy(datagrid, false);

            //Disabilito l'aggiunta di un record nella griglia
            abilitaModificaRecord(datagrid, true);

            //Disabilito l'eliminazione dei record.
            abilitaEliminazioneRecord(datagrid, false);

            //Abilito i filtri della griglia.
            abilitaFiltriDataGrid(datagrid, true);

            //Abilito il layer di gestione degli error (IDataError)
            abilitaLayerErrorDataGrid(datagrid);

            //Abilito il copia incolla.
            abilitaCopiaIncolla(datagrid, CopyCommand);

            //Imposto se posso cambiare la visualizzazione delle colonne della griglia.
            selezioneCampiVisibiliGriglia(datagrid, false);
        }

        public static void setProprietaPannelliPianoStudio(XamDataGrid datagrid)
        {
            //Settaggi del datagrid generico
            impostaTemaDataGrid(datagrid);

            //Abilito l'auto fit delle colonne
            abilitaAutoFit(datagrid, true);

            //Abilito la selezione con l'elemento corrente
            abilitaSincronizzazioneConElementoCorrente(datagrid, true);

            //Abilito il Drag And Drop
            abilitaDragAndDrop(datagrid, true);

            //Imposto la funzione per la selezione di una cella
            funzioneSelezioneCella(datagrid, CellClickAction.EnterEditModeIfAllowed);

            //Imposto la funzione per la selezione di un header.
            funzioneSelezioneLabelCella(datagrid, LabelClickAction.SortByMultipleFields);

            //Abilito il sommario
            abilitaSommario(datagrid);

            //Abilita la selezione multipla degli elementi.
            abilitaSelezioneMultipla(datagrid, true);

            //Disabilito la generazione automatica dei campi nella griglia
            abilitaAutoGenerazioneCampi(datagrid, false);

            //Abilito il group by nella griglia
            abilitaGroupBy(datagrid, true);

            //Abilito la stilizzazione dell'header del Datagrid.
            //abilitaHeaderDataGrid(datagrid);

            //Disabilito l'aggiunta di un record nella griglia
            abilitaModificaRecord(datagrid, true);

            //Disabilito l'eliminazione dei record.
            abilitaEliminazioneRecord(datagrid, false);

            //Abilito i filtri della griglia.
            abilitaFiltriDataGrid(datagrid, true);

            //Abilito il copia incolla.
            abilitaCopiaIncolla(datagrid, CopyCommand);

            //Imposto se posso cambiare la visualizzazione delle colonne della griglia.
            selezioneCampiVisibiliGriglia(datagrid, false);
        }

        public static void setProprietaPannelliInterni(XamDataGrid datagrid)
        {
            //Settaggi del datagrid generico
            impostaTemaDataGrid(datagrid);

            //Abilito l'auto fit delle colonne
            abilitaAutoFit(datagrid, true);

            //Abilito la selezione con l'elemento corrente
            abilitaSincronizzazioneConElementoCorrente(datagrid, true);

            //Abilito il Drag And Drop
            abilitaDragAndDrop(datagrid, true);

            //Imposto la funzione per al momento della selezione di una cella
            funzioneSelezioneCella(datagrid, CellClickAction.EnterEditModeIfAllowed);

            //Abilito il sommario
            abilitaSommario(datagrid);

            //Disabilito la selezione multipla
            abilitaSelezioneMultipla(datagrid, false);

            //Disabilito la generazione automatica dei campi nella griglia
            abilitaAutoGenerazioneCampi(datagrid, false);

            //Disabilito il group by nella griglia
            abilitaGroupBy(datagrid, false);

            //Disabilito l'aggiunta di un record nella griglia
            abilitaModificaRecord(datagrid, true);

            //Disabilito l'eliminazione dei record.
            abilitaEliminazioneRecord(datagrid, false);

            //Abilito i filtri della griglia.
            abilitaFiltriDataGrid(datagrid, true);

            //Abilito il layer di gestione degli error (IDataError)
            abilitaLayerErrorDataGrid(datagrid);

            //Abilito il copia incolla.
            abilitaCopiaIncolla(datagrid, CopyCommand);

            //Imposto se posso cambiare la visualizzazione delle colonne della griglia.
            selezioneCampiVisibiliGriglia(datagrid, false);
        }

        public static void setProprietaContatti(XamDataGrid datagrid)
        {
            //Settaggi del datagrid generico
            //impostaTemaDataGrid(datagrid);

            //Disabilito il group by nella griglia
            abilitaGroupBy(datagrid, false);

            //Abilito la modifica di un record nella griglia
            abilitaModificaRecord(datagrid, true);

            //Abilito il copia incolla.
            abilitaCopiaIncolla(datagrid, CopyCommand);
        }
    }
}
