using System;
using System.ComponentModel;

namespace HeaderProblem
{
    [Serializable]
    public class PersonEntity : ABusinessObjectEntity, INotifyPropertyChanged, IDataErrorInfo
    {

        private int _persID;
        private string _cognome = "";
        private string _nome = "";
        private string _datanascita = "";
        private bool   _datanascitavalida = true;
        private int    _sesso;
        private string _sessoFK = "";
        private string _cortesia = "";
        private string _commento = "";
        private int    _dipartimento;
        private int    _cortesiaID;
        private string _cortesiaFK = "";
        private string _cittadinanza = "";
        private string _doppiacittadinanza = "";
        private string _attinenza = "";

        /**
         * Costruttore di default della classe PersonEntity
         **/
        public PersonEntity()
        {
        }

        /**
         * Costruttore specifico della classe PersonEntity
         * Parametri in entrata
         * @cognome
         * @nome
         * @datanascita
         * 
         * Identificano le tre informazioni per identificare univocamente una persona.
         **/
        public PersonEntity(String cognome, String nome, String datanascita)
        {
            _persID = 0;
            _cognome = cognome;
            _nome = nome;
            _datanascita = datanascita;
        }

  
        /**
         * Propriet che contiene l'ID della persona.
         **/
        public int persID
        {
            get { return _persID; }
            set
            {
                _persID = value;
                NotifyPropertyChanged("persID");
            }
        }

        /**
         * Propriet che contiene l'ID del dipartimento.
         **/
        public int dipartimento
        {
            get { return _dipartimento; }
            set
            {
                _dipartimento = value;
                NotifyPropertyChanged("dipartimento");
            }
        }

        /**
         * Propriet che contiene il cognome della persona.
         **/
        public string cognome
        {
            get { return _cognome; }
            set
            {
                _cognome = value.Trim();
                NotifyPropertyChanged("cognome");
            }
        }

        /**
         * Propriet che contiene il nome della persona.
         **/
        public string nome
        {
            get { return _nome; }
            set
            {
                _nome = value.Trim();
                NotifyPropertyChanged("nome");
            }
        }

        /**
         * Propriet che contiene la data di nascita della persona.
         **/
        public string datanascita
        {
            get { return _datanascita; }
            set
            {
                if (value.Trim() == "")
                {
                    _datanascita = value;
                    _datanascitavalida = true;
                    NotifyPropertyChanged("datanascita");
                    return;
                }

                try
                {
                    DateTime data = Convert.ToDateTime(value);
                    _datanascita = data.ToShortDateString();
                    _datanascitavalida = true;
                    NotifyPropertyChanged("datanascita");
                }
                catch (Exception)
                {
                    _datanascita = "";
                    _datanascitavalida = false;
                    NotifyPropertyChanged("datanascita");
                }
            }
        }

        /**
         * Propriet che contiene l'ID del dipartimento.
         **/
        public int sesso
        {
            get { return _sesso; }
            set
            {
                _sesso = value;
                NotifyPropertyChanged("sesso");
            }
        }

        /**
         * Propriet che contiene l'ID del dipartimento.
         **/
        public string sessoFK
        {
            get { return _sessoFK; }
            set
            {
                _sessoFK = value;
            }
        }

        /**
         * Propriet che contiene la stringa di cortesia della persona.
         **/
        public string cortesia
        {
            get
            {
                return _cortesia;
            }
            set
            {
                _cortesia = value.Trim();
                NotifyPropertyChanged("cortesia");
            }
        }

        /**
         * Propriet che contiene l'ID del dipartimento.
         **/
        public string cortesiaFK
        {
            get { return _cortesiaFK; }
            set
            {
                _cortesiaFK = value;
            }
        }

        /**
         * Propriet che contiene il commento sulla persona.
         **/
        public string commento
        {
            get
            {
                return _commento;
            }
            set
            {
                _commento = value.Trim();
                NotifyPropertyChanged("commento");
            }
        }
        /**
         * Propriet che contiene l'ID della persona.
         **/
        public int cortesiaID
        {
            get { return _cortesiaID; }
            set
            {
                _cortesiaID = value;
                NotifyPropertyChanged("cortesiaID");
            }
        }

        /**
         * Propriet che contiene l'attinenza dello studente.
         **/
        public string attinenza
        {
            get { return _attinenza; }
            set
            {
                _attinenza = value.Trim();
                NotifyPropertyChanged("attinenza");
            }
        }

        /**
         * Propriet che contiene la doppia cittadinanza dello studente.
         **/
        public string doppiacittadinanza
        {
            get { return _doppiacittadinanza; }
            set
            {
                _doppiacittadinanza = value.Trim();
                NotifyPropertyChanged("doppiacittadinanza");
            }
        }

        /**
         * Propriet che contiene la cittadinanza dello studente.
         **/
        public string cittadinanza
        {
            get { return _cittadinanza; }
            set
            {
                _cittadinanza = value.Trim();
                NotifyPropertyChanged("cittadinanza");
            }
        }

        /**
         * Funzione che aggiorna lo stato dell'oggetto persona.
         **/

        /**
         * Funzione che controllo lo stato di integrit dell'oggetto persona.
         **/
        public override bool oggettoIntegro()
        {
            oggettoCompleto = true;
            oggettoCompleto &= (!string.IsNullOrEmpty(_nome) && _nome.Trim().Length <= 50); //Dato obbligatorio
            oggettoCompleto &= (!string.IsNullOrEmpty(_cognome) && _cognome.Trim().Length <= 50); //Dato obbligatorio
            oggettoCompleto &= _datanascitavalida; //Data obbligatoria
            oggettoCompleto &= _attinenza.Trim().Length <= 100; //Dato facoltativo
            oggettoCompleto &= _cittadinanza.Trim().Length <= 100; //Dato facoltativo
            oggettoCompleto &= _doppiacittadinanza.Trim().Length <= 100; //Dato facoltativo
            oggettoCompleto &= _commento.Trim().Length <= 4000; //Dato facoltativo

            return oggettoCompleto;
        }

        public override ABusinessObjectDB getDBHandler()
        {
            throw new NotImplementedException();
        }

        public override bool Equals(object obj)
        {
            var oggetto = obj as PersonEntity;
            if (obj == null || oggetto == null)
                return false;

            return (oggetto.persID == persID && oggetto.tipoOggetto == tipoOggetto && oggetto.timestamp == timestamp);
        }

        public override int GetHashCode()
        {
            return (persID + tipoOggetto + timestamp).GetHashCode();
        }


        public override void copiaOggetto(ABusinessObjectEntity oggettoDaCopiare)
        {
            throw new NotImplementedException();
        }


        // Declare the PropertyChanged event
        [field: NonSerialized]
        public event PropertyChangedEventHandler PropertyChanged;

        // NotifyPropertyChanged will raise the PropertyChanged event passing the
        // source property that is being updated.
        public void NotifyPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }


        #region IDataErrorInfo Members

        public string Error
        {
            get { return null; }
        }

        public string this[string property]
        {
            get
            {
                string msg = null;
                switch (property)
                {
                    case "persID":
                        if (persID < 0) 
                        {
                            msg = "Il pers_id non  valido.";
                        }else if(persID > 99999999)
                        {
                            msg = "Il pers_id non pu essere maggiore di 10 cifre.";
                        }
                        break;
                    case "cortesiaID":
                        if (cortesiaID < 0)
                        {
                            msg = "La cortesia non  valida.";
                        }
                        break;
                    case "cognome":
                        //Verifico se il campo obbligatorio inserito non  vuoto.
                        if (cognome.Trim() == "")
                        {
                            msg = "Il cognome  un campo obbligatorio.";
                        }
                        //Verifico la lunghezza massima del campo.
                        if (cognome.Trim().Length > 50)
                        {
                            msg = "Il cognome pu essere lungo al massimo 50 caratteri.";
                        }
                        break;
                    case "nome":
                        //Verifico se il campo obbligatorio inserito non  vuoto.
                        if (nome.Trim() == "")
                        {
                            msg = "Il nome  un campo obbligatorio.";
                        }
                        //Verifico la lunghezza massima del campo.
                        if (nome.Trim().Length > 50)
                        {
                            msg = "Il nome pu essere lungo al massimo 50 caratteri.";
                        }
                        break;
                    case "datanascita":
                        if (datanascita.Trim() == "" && !_datanascitavalida)
                        {
                            msg = "Il formato della data non  valido.";
                        }
                        break;
                    case "commento":
                        //Verifico la lunghezza massima del campo.
                        if (commento.Trim().Length > 4000)
                        {
                            msg = "Il commento pu essere lungo al massimo 4000 caratteri.";
                        }
                        break;
                    case "sesso":
                        if (sesso < 0)
                        {
                            msg = "Il sesso della persona non  valido.";
                        }
                        if (sesso > 999999)
                        {
                            msg = "Il sesso della persona  non  valido.";
                        }
                        break;
                    case "attinenza":
                        //Verifico la lunghezza massima del campo.
                        if (attinenza.Trim().Length > 100)
                        {
                            msg = "L'attinenza pu essere lunga al massimo 100 caratteri.";
                        }
                        break;
                    case "cittadinanza":
                        //Verifico la lunghezza massima del campo.
                        if (cittadinanza.Trim().Length > 100)
                        {
                            msg = "La cittadinanza pu essere lunga al massimo 100 caratteri.";
                        }
                        break;
                    case "doppiacittadinanza":
                        //Verifico la lunghezza massima del campo.
                        if (doppiacittadinanza.Trim().Length > 100)
                        {
                            msg = "La seconda cittadinanza pu essere lunga al massimo 100 caratteri.";
                        }
                        break; 
                    case "dipartimento":
                        if (dipartimento < 0)
                        {
                            msg = "Il dipartimento  errato.";
                        }else if(dipartimento > 999999)
                        {
                            msg = "Il dipartimento non pu essere maggiore di 6 cifre.";
                        }
                        break; 

                    default:
                        throw new ArgumentException("Propriet non riconosciuta: " + property);
                }
                return msg;
            }
        }

        #endregion

    }
}
