﻿
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Data;
using System.Windows.Media;
using Infragistics.Windows.Controls;
using Infragistics.Windows.DataPresenter;

namespace HeaderProblem
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1
    {
        public Window1()
        {
            InitializeComponent();

            //Datagrid 1
            setProperty(xamDataGrid1);
            loadDataWithHeader();

            //Datagrid 2
            setProperty(xamDataGrid2);
            loadDataWithoutHeaderAndNoData();

            //Datagrid 3
            setProperty(xamDataGrid3);
            loadDataWithHeaderAndNoDataAndSummaryPositionCommented(); //Datagrid 3

            //For information
            //ABusinessObjectEntity is an abstract class
            //PersonEntity : ABusinessObjectEntity
        }

        private void loadDataWithHeader()
        {
            var campiVisibili = new List<FieldSDB>
                                 {
                                     generaCampo("1", "cognome", false, Visibility.Visible),
                                     generaCampo("2", "nome", false, Visibility.Visible)
                                 };

            generaCampiDataGrid(campiVisibili, xamDataGrid1);

            xamDataGrid1.FieldSettings.SummaryDisplayArea = SummaryDisplayAreas.BottomFixed;

            impostaSommario(xamDataGrid1, "cognome", "1");

            var lista = new ObservableCollection<ABusinessObjectEntity>();

            var person = new PersonEntity {cognome = "Franklin", nome = "Benjamin"};
            lista.Add(person);

            xamDataGrid1.DataSource = lista;
        }

        private void loadDataWithoutHeaderAndNoData()
        {
            var campiVisibili = new List<FieldSDB>
                                 {
                                     generaCampo("1", "cognome", false, Visibility.Visible),
                                     generaCampo("2", "nome", false, Visibility.Visible)
                                 };

            generaCampiDataGrid(campiVisibili, xamDataGrid2);

            xamDataGrid2.FieldSettings.SummaryDisplayArea = SummaryDisplayAreas.BottomFixed;

            impostaSommario(xamDataGrid2, "cognome", "1");

            var lista = new ObservableCollection<ABusinessObjectEntity>();

            xamDataGrid2.DataSource = lista;
        }

        private void loadDataWithHeaderAndNoDataAndSummaryPositionCommented()
        {
            var campiVisibili = new List<FieldSDB>
                                 {
                                     generaCampo("1", "cognome", false, Visibility.Visible),
                                     generaCampo("2", "nome", false, Visibility.Visible)
                                 };

            generaCampiDataGrid(campiVisibili, xamDataGrid3);

            //===========================================================
            //Commented. Without this code line the header appears !!!!!!
            //===========================================================
            //xamDataGrid3.FieldSettings.SummaryDisplayArea = SummaryDisplayAreas.BottomFixed;

            impostaSommario(xamDataGrid3, "cognome", "1");

            var lista = new ObservableCollection<ABusinessObjectEntity>();

            xamDataGrid3.DataSource = lista;
        }

        private static FieldSDB generaCampo(string idCampo, string binding, bool solaLettura, Visibility visibilita)
        {
            var campo = new FieldSDB
            {
                bindingCampo = binding,
                solaLettura = solaLettura,
                visibilitaCampo = visibilita
            };

            switch(idCampo)
            {
                case "1":
                    campo.unboundCampo = true;
                    campo.nomeCampo = "cognome";
                    campo.labelCampo = "cognome";
                    break;
                case "2":
                    campo.unboundCampo = true;
                    campo.nomeCampo = "nome";
                    campo.labelCampo = "nome";
                    break;
            }

            return campo;
        }

        public static void impostaSommario(XamDataGrid datagrid, string nomeCampo, string tipoSommario)
        {
            var sommario = new SummaryDefinition();

            switch (tipoSommario)
            {
                case "1":
                    sommario.Calculator = SummaryCalculator.Count;
                    break;
                default:
                    break;
            }

            sommario.SourceFieldName = nomeCampo;

            if (datagrid.FieldLayouts.Count == 1)
            {
                datagrid.FieldLayouts[0].SummaryDefinitions.Add(sommario);
            }
            else
            {
                //Definizione del sommario
                var fieldlayout = new FieldLayout();
                fieldlayout.SummaryDefinitions.Add(sommario);
                datagrid.FieldLayouts.Add(fieldlayout);
            }
        }

        private void setProperty(XamDataGrid datagrid)
        {
            //Settaggi del datagrid generico
            impostaTemaDataGrid(datagrid);

            //Abilito l'auto fit delle colonne
            abilitaAutoFit(datagrid, true);

            //Disabilito il Drag And Drop
            abilitaDragAndDrop(datagrid, false);

            //Imposto la funzione per al momento della selezione di una cella
            funzioneSelezioneCella(datagrid, CellClickAction.SelectRecord);

            //Abilito la selezione dell'ordinamento sull'header della colonna
            funzioneSelezioneLabelCella(datagrid, LabelClickAction.SortByMultipleFields);

            //Abilito la selezione con l'elemento corrente
            abilitaSincronizzazioneConElementoCorrente(datagrid, true);

            //Disabilito la generazione automatica dei campi nella griglia
            abilitaAutoGenerazioneCampi(datagrid, false);

            //Disabilito il group by nella griglia
            abilitaGroupBy(datagrid, false);

            //Abilito il sommario
            abilitaSommario(datagrid);

            //Disabilito l'aggiunta di un record nella griglia
            abilitaModificaRecord(datagrid, false);

            //Disabilito l'eliminazione dei record.
            abilitaEliminazioneRecord(datagrid, true);

            //Abilito i filtri della griglia.
            abilitaFiltriDataGrid(datagrid, true);

            //Imposto se posso cambiare la visualizzazione delle colonne della griglia.
            selezioneCampiVisibiliGriglia(datagrid, false);
        }

        #region Gestione impostazioni di base dei xamDataGrid1

        public static void impostaTemaDataGrid(XamDataGrid xamDataGrid1)
        {
            xamDataGrid1.Theme = "Office2k7Blue";
            xamDataGrid1.FontFamily = new FontFamily("Calibri");
            xamDataGrid1.FontSize = 12;
            xamDataGrid1.FieldSettings.Height = new FieldLength(21);

        }

        public static void abilitaSincronizzazioneConElementoCorrente(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.IsSynchronizedWithCurrentItem = enabled;
        }

        public static void funzioneSelezioneCella(XamDataGrid xamDataGrid1, CellClickAction funzione)
        {
            xamDataGrid1.FieldSettings.CellClickAction = funzione;
        }

        public static void funzioneSelezioneLabelCella(XamDataGrid xamDataGrid1, LabelClickAction funzione)
        {
            xamDataGrid1.FieldSettings.LabelClickAction = funzione;
        }

        public static void abilitaSelezioneMultipla(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.FieldLayoutSettings.SelectionTypeRecord = enabled ? SelectionType.Extended : SelectionType.Single;
        }

        public static void selezioneCampiVisibiliGriglia(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.FieldLayoutSettings.HeaderPrefixAreaDisplayMode = enabled ? HeaderPrefixAreaDisplayMode.FieldChooserButton : HeaderPrefixAreaDisplayMode.None;
        }

        public static void abilitaDragAndDrop(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.AllowDrop = enabled;
        }

        public static void abilitaAggiuntaRecord(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.FieldLayoutSettings.AllowAddNew = enabled;
            xamDataGrid1.FieldLayoutSettings.AddNewRecordLocation = AddNewRecordLocation.OnTopFixed;
        }

        public static void abilitaModificaRecord(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.FieldSettings.AllowEdit = enabled;
        }

        public static void abilitaEliminazioneRecord(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.FieldLayoutSettings.AllowDelete = enabled;
        }

        public static void abilitaAutoGenerazioneCampi(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.FieldLayoutSettings.AutoGenerateFields = enabled;
        }

        public static void abilitaAutoFit(XamDataGrid xamDataGrid1, bool enabled)
        {
            xamDataGrid1.AutoFit = enabled;
        }

        public static void abilitaGroupBy(XamDataGrid xamDataGrid1, bool enabled)
        {
            if (enabled)
            {
                xamDataGrid1.FieldSettings.AllowGroupBy = enabled;
                xamDataGrid1.GroupByAreaLocation = GroupByAreaLocation.AboveDataArea;
                xamDataGrid1.GroupByAreaMode = GroupByAreaMode.MultipleFieldLayoutsCompact;
                xamDataGrid1.IsGroupByAreaExpanded = false;
                xamDataGrid1.GroupByArea.Prompt1 = "Area di raggruppamento";
                xamDataGrid1.GroupByArea.Prompt2 = "Trascina un campo";
            }
            else
            {
                xamDataGrid1.FieldSettings.AllowGroupBy = enabled;
                xamDataGrid1.GroupByAreaLocation = GroupByAreaLocation.None;
            }
        }

        public static void abilitaFiltriDataGrid(XamDataGrid xamDataGrid1, bool abilita)
        {
            if (abilita)
            {
                xamDataGrid1.FieldSettings.AllowRecordFiltering = true;
                xamDataGrid1.FieldSettings.FilterClearButtonVisibility = Visibility.Visible;
                xamDataGrid1.FieldLayoutSettings.FilterUIType = FilterUIType.LabelIcons;
                xamDataGrid1.FieldLayoutSettings.DataRecordSizingMode = DataRecordSizingMode.SizedToContentAndFixed;
                xamDataGrid1.FieldLayoutSettings.RecordFiltersLogicalOperator = LogicalOperator.And;
                xamDataGrid1.FieldLayoutSettings.FilterClearButtonLocation = FilterClearButtonLocation.RecordSelector;
            }
            else
            {
                xamDataGrid1.FieldSettings.AllowRecordFiltering = false;
            }
        }

        public static void abilitaLayerErrorDataGrid(XamDataGrid xamDataGrid1)
        {
            xamDataGrid1.FieldLayoutSettings.SupportDataErrorInfo = SupportDataErrorInfo.RecordsAndCells;
            xamDataGrid1.FieldLayoutSettings.DataErrorDisplayMode = DataErrorDisplayMode.ErrorIconAndHighlight;
        }

        public static void abilitaSommario(XamDataGrid datagrid)
        {
            datagrid.FieldSettings.AllowSummaries = false;
            datagrid.FieldSettings.SummaryUIType = SummaryUIType.SingleSelect;
            //datagrid.FieldSettings.SummaryDisplayArea = SummaryDisplayAreas.BottomFixed;
        }

        #endregion


        #region Generazione campi della griglia

        public static void generaCampiDataGrid(IList<FieldSDB> campiVisibili, DataPresenterBase datagrid)
        {
            //Svuoto tutti i campi.
            datagrid.FieldLayouts[0].Fields.Clear();
            datagrid.FieldLayouts[0].SummaryDefinitions.Clear();

            for (int i = 0; i < campiVisibili.Count; i++)
            {
                if (!campiVisibili[i].unboundCampo)
                {
                    var field = new Field
                    {
                        Name = campiVisibili[i].nomeCampo,
                        Label = campiVisibili[i].labelCampo,
                        Visibility = campiVisibili[i].visibilitaCampo
                    };

                    if (campiVisibili[i].dimensioneCampo != 0.0)
                    {
                        field.Width = new FieldLength(campiVisibili[i].dimensioneCampo);
                    }

                    if (campiVisibili[i].tipoCampo != null)
                    {
                        field.DataType = campiVisibili[i].tipoCampo;
                        field.Settings.EditAsType = campiVisibili[i].tipoCampo;
                    }

                    if (!string.IsNullOrEmpty(campiVisibili[i].stileCellaCampo))
                    {
                        field.Settings.CellValuePresenterStyle = (Style)datagrid.TryFindResource(campiVisibili[i].stileCellaCampo);
                    }

                    if (!string.IsNullOrEmpty(campiVisibili[i].editoreCampo))
                    {
                        field.Settings.EditorStyle = (Style)datagrid.TryFindResource(campiVisibili[i].editoreCampo);
                    }

                    if (campiVisibili[i].solaLettura)
                    {
                        field.Settings.EditorStyle = (Style)datagrid.TryFindResource("ReadOnlyFieldStyle");
                        field.Settings.AllowEdit = false;
                    }

                    field.IsScrollTipField = campiVisibili[i].toolTipScroll;

                    if (campiVisibili[i].campoGroupBy)
                    {
                        field.IsExpandable = true;
                    }

                    datagrid.FieldLayouts[0].Fields.Add(field);
                }
                else
                {
                    var pp = new PropertyPath(campiVisibili[i].bindingCampo);

                    var ufield = new UnboundField
                    {
                        Name = campiVisibili[i].nomeCampo,
                        Label = campiVisibili[i].labelCampo,
                        Visibility = campiVisibili[i].visibilitaCampo,
                        BindingPath = pp,
                        BindingMode = BindingMode.TwoWay
                    };


                    if (campiVisibili[i].dimensioneCampo != 0)
                    {
                        ufield.Width = new FieldLength(campiVisibili[i].dimensioneCampo);
                    }

                    if (campiVisibili[i].tipoCampo != null)
                    {
                        ufield.DataType = campiVisibili[i].tipoCampo;
                        ufield.Settings.EditAsType = campiVisibili[i].tipoCampo;
                    }

                    if (!string.IsNullOrEmpty(campiVisibili[i].stileCellaCampo))
                    {
                        ufield.Settings.CellValuePresenterStyle = (Style)datagrid.TryFindResource(campiVisibili[i].stileCellaCampo);
                    }

                    if (!string.IsNullOrEmpty(campiVisibili[i].editoreCampo))
                    {
                        ufield.Settings.EditorStyle = (Style)datagrid.TryFindResource(campiVisibili[i].editoreCampo);

                    }

                    if (campiVisibili[i].solaLettura)
                    {
                        ufield.Settings.EditorStyle = (Style)datagrid.TryFindResource("ReadOnlyFieldStyle");
                        ufield.Settings.AllowEdit = false;
                    }

                    ufield.IsScrollTipField = campiVisibili[i].toolTipScroll;

                    if (campiVisibili[i].campoGroupBy)
                    {
                        ufield.IsExpandable = true;
                    }

                    datagrid.FieldLayouts[0].Fields.Add(ufield);
                }
            }
        }

        #endregion
    }
}
